import { world, system, ItemStack, Vector, EquipmentSlot, ItemLockMode } from '@minecraft/server';
import { config, sendAllMessage, sendPlayerMessage } from './config';
import { JaylyDB } from './database';
export var MinecraftEffectTypes;
(function (MinecraftEffectTypes) {
    MinecraftEffectTypes["Absorption"] = "absorption";
    MinecraftEffectTypes["BadOmen"] = "bad_omen";
    MinecraftEffectTypes["Blindness"] = "blindness";
    MinecraftEffectTypes["ConduitPower"] = "conduit_power";
    MinecraftEffectTypes["Darkness"] = "darkness";
    MinecraftEffectTypes["Empty"] = "empty";
    MinecraftEffectTypes["FatalPoison"] = "fatal_poison";
    MinecraftEffectTypes["FireResistance"] = "fire_resistance";
    MinecraftEffectTypes["Haste"] = "haste";
    MinecraftEffectTypes["HealthBoost"] = "health_boost";
    MinecraftEffectTypes["Hunger"] = "hunger";
    MinecraftEffectTypes["InstantDamage"] = "instant_damage";
    MinecraftEffectTypes["InstantHealth"] = "instant_health";
    MinecraftEffectTypes["Invisibility"] = "invisibility";
    MinecraftEffectTypes["JumpBoost"] = "jump_boost";
    MinecraftEffectTypes["Levitation"] = "levitation";
    MinecraftEffectTypes["MiningFatigue"] = "mining_fatigue";
    MinecraftEffectTypes["Nausea"] = "nausea";
    MinecraftEffectTypes["NightVision"] = "night_vision";
    MinecraftEffectTypes["Poison"] = "poison";
    MinecraftEffectTypes["Regeneration"] = "regeneration";
    MinecraftEffectTypes["Resistance"] = "resistance";
    MinecraftEffectTypes["Saturation"] = "saturation";
    MinecraftEffectTypes["SlowFalling"] = "slow_falling";
    MinecraftEffectTypes["Slowness"] = "slowness";
    MinecraftEffectTypes["Speed"] = "speed";
    MinecraftEffectTypes["Strength"] = "strength";
    MinecraftEffectTypes["VillageHero"] = "village_hero";
    MinecraftEffectTypes["WaterBreathing"] = "water_breathing";
    MinecraftEffectTypes["Weakness"] = "weakness";
    MinecraftEffectTypes["Wither"] = "wither";
})(MinecraftEffectTypes || (MinecraftEffectTypes = {}));
import './combatlog/commands/staffOnly/combattime';
import './combatlog/commands/rsp';
import './combatlog/commands/staffOnly/sptime';
import './combatlog/commands/check';
import './combatlog/commands/staffOnly/mobtriggerlog';
import './combatlog/commands/help';
import './combatlog/commands/staffOnly/itemdroplog';
import './combatlog/commands/staffOnly/disableflycombat';
export function getPlayer(playerName) {
    return world.getAllPlayers().find(player => player.name.toLowerCase() === playerName.toLowerCase().replace(/"|\\|@/g, ""));
}
export const clSettings = new JaylyDB(`clsettings`);
const defaultSettings = {
    combattime: `30`,
    spawnkilltime: `30`,
    mobtriggerlog: `false`,
    disabletpcombat: `false`,
    itemdroplog: `false`,
    disableelytracombat: `false`
};
for (const key in defaultSettings) {
    if (!clSettings.has(key)) {
        clSettings.set(key, defaultSettings[key]);
    }
}
export function getScore(player, objective) {
    let score = world.scoreboard.getObjective(objective)?.getScore(player.scoreboardIdentity);
    return score;
}
let invArr = [];
world.afterEvents.playerLeave.subscribe(({ playerName }) => {
    let x = invArr.find(x => x.playerName == playerName);
    if (x.logged == false || clSettings.get(`itemdroplog`) === `false`)
        return;
    try {
        world.getDimension(x.dimension).spawnItem(new ItemStack(`lifesteal:heart`), x.location);
    }
    catch { }
    for (let item of x.inventory) {
        world.getDimension(x.dimension).spawnItem(item, x.location);
    }
});
system.runInterval(() => {
    for (let player of world.getAllPlayers()) {
        let equipmentInventory = player.getComponent(`equipment_inventory`);
        let plr = invArr.find(x => x.player == player);
        if (player.hasTag(`cl-combatlogged`) && clSettings.get(`disabletpcombat`) === `true` && plr) {
            if (Vector.distance(plr.location, player.location) > 40) {
                player.teleport(plr.location, { dimension: player.dimension });
                sendPlayerMessage(player, `§4You are in combat and have been teleported back!`);
            }
        }
        let inventory = player.getComponent(`inventory`);
        let items = [];
        const equipments = {
            headItem: equipmentInventory.getEquipment(EquipmentSlot.head),
            chestItem: equipmentInventory.getEquipment(EquipmentSlot.chest),
            legItem: equipmentInventory.getEquipment(EquipmentSlot.legs),
            feetItem: equipmentInventory.getEquipment(EquipmentSlot.feet),
            offhandItem: equipmentInventory.getEquipment(EquipmentSlot.offhand)
        };
        for (const key in equipments) {
            let item = equipments[key];
            if (!item)
                continue;
            items.push(item);
        }
        for (let i = 0; i < inventory.container.size; i++) {
            let item = inventory.container.getItem(i);
            if (!item)
                continue;
            items.push(item);
        }
        let { name: playerName, location, dimension } = player;
        let logged = player.getTags().includes(`cl-combatlogged`);
        let index = invArr.findIndex(x => x.player === player);
        let obj = { playerName, location, dimension: dimension.id, inventory: items, logged, player };
        index !== -1 ? invArr[index] = obj : invArr.push(obj);
    }
}, 10);
world.afterEvents.entityDie.subscribe(data => {
    let player = data.deadEntity;
    let damaging = data.damageSource.damagingEntity;
    try {
        player.typeId;
    }
    catch {
        return;
    }
    if (player.typeId !== `minecraft:player`)
        return;
    if (player.hasTag(`cl-combatlogged`))
        player.runCommand(`title @s actionbar §3You are no longer in combat!`);
    player.removeTag(`cl-combatlogged`);
    if (!damaging || damaging.typeId !== `minecraft:player` || player == damaging)
        return;
    system.runTimeout(() => {
        player.addTag(`cl-spawnkill`);
    }, 5);
});
world.afterEvents.entityHurt.subscribe(data => {
    let player = data.hurtEntity;
    let damaging = data.damageSource.damagingEntity;
    let health = player.getComponent(`health`);
    if (!damaging)
        return;
    if (player.hasTag(`cl-spawnkill`)) {
        if (player !== damaging && damaging.typeId == `minecraft:player`) {
            sendPlayerMessage(damaging, `§cYou cannot damage this player as they have just been killed!`);
            for (let i = 0; i < 10; i++) {
                player.applyKnockback(0, 0, 0, 0);
                player.applyKnockback(0, 0, 0, 0);
            }
            health.setCurrentValue(Math.ceil(health.currentValue + data.damage));
            damaging.addEffect(`weakness`, 40, { amplifier: 255, showParticles: false });
            system.runTimeout(() => {
                if (health.currentValue <= 0) {
                    damaging.kill();
                    sendAllMessage(`§cKilled ${damaging.name} for §6Spawn Killing`);
                }
            }, 1);
        }
        return;
    }
    if (damaging.hasTag(`cl-spawnkill`)) {
        player.runCommand(`title @s actionbar §cYou no longer have spawn protection!`);
        player.removeTag(`cl-spawnkill`);
    }
    if (player == damaging)
        return;
    if (clSettings.get(`mobtriggerlog`) == `false` && (player.typeId !== `minecraft:player` || damaging.typeId !== `minecraft:player`))
        return;
    if (clSettings.get('mobtriggerlog') == `true`) {
        let isPlayer = (entity) => entity.typeId === `minecraft:player`;
        let isMonster = (entity) => Array.from(entity.dimension.getEntities({ type: entity.typeId, families: [`monster`] })).length !== 0;
        let bothPlayers = isPlayer(player) && isPlayer(damaging);
        let oneMonster = isMonster(player) || isMonster(damaging);
        if (!bothPlayers && !oneMonster)
            return;
    }
    let combatscore = +clSettings.get(`combattime`);
    if (!player.hasTag(`cl-combatlogged`))
        sendPlayerMessage(player, `§8[§6!§8] §4You are now combat logged! §6Do not leave or you will die §l(§3${combatscore >= 60 ? `${Math.floor(combatscore / 60)} ${Math.floor(combatscore / 60) == 1 ? `minute` : `minutes`} ${Math.floor(combatscore - Math.floor(combatscore / 60) * 60)}` : `${Math.floor(combatscore)}`}s§6)`);
    if (player.typeId == `minecraft:player`) {
        player.addTag(`cl-combatlogged`);
        player.runCommand(`scoreboard players set @s combatlog ${clSettings.get(`combattime`) * 20}`);
    }
    if (damaging.typeId == `minecraft:player`) {
        damaging.addTag(`cl-combatlogged`);
        damaging.runCommand(`scoreboard players set @s combatlog ${clSettings.get(`combattime`) * 20}`);
    }
});
function formatTime(time) {
    let minutes = Math.floor((time / 20) / 60);
    let seconds = Math.floor(time / 20) - (minutes * 60);
    return time / 20 >= 60 ? `${minutes ?? 0} minute${minutes === 1 ? `` : `s`} ${seconds ?? 0}s` : `${seconds ?? 0}s`;
}
function stopGliding(player) {
    let equipmentInventory = player.getComponent(`equipment_inventory`);
    let elytra = equipmentInventory.getEquipment(EquipmentSlot.chest);
    let barrier = new ItemStack(`barrier`);
    barrier.lockMode = ItemLockMode.slot;
    barrier.nameTag = `§cSubscribe to GamerDos on YT`;
    equipmentInventory.setEquipment(EquipmentSlot.chest, barrier);
    player.applyKnockback(0, 0, 0, 0);
    player.applyKnockback(0, 0, 0, 0);
    player.applyKnockback(0, 0, 0, 0);
    player.applyKnockback(0, 0, 0, 0);
    player.applyKnockback(0, 0, 0, 0);
    system.runTimeout(() => {
        equipmentInventory.setEquipment(EquipmentSlot.chest, elytra);
    }, 10);
    sendPlayerMessage(player, `§4You are in combat and you cannot use your elytra!`);
}
system.runInterval(() => {
    let players = world.getAllPlayers();
    for (let player of players) {
        if (player.hasTag(`cl-combatlogged`) && clSettings.get(`disableelytracombat`) === `true` && player.isGliding) {
            stopGliding(player);
        }
        if (!player.hasTag(`cl-combatlogged`))
            player.runCommand(`scoreboard players set @s combatlog ${clSettings.get(`combattime`) * 20}`);
        else {
            let combatscore = getScore(player, `combatlog`);
            player.runCommand(`scoreboard players remove @s combatlog 1`);
            player.runCommand(`title @s actionbar §eYou are §ccombat logged! §l§3(§p${formatTime(combatscore)}§3)`);
        }
        if (!player.hasTag(`cl-spawnkill`))
            player.runCommand(`scoreboard players set @s spawnkill ${clSettings.get(`spawnkilltime`) * 20}`);
        else {
            player.removeTag(`cl-combatlogged`);
            let spscore = getScore(player, `spawnkill`);
            player.runCommand(`scoreboard players remove @s spawnkill 1`);
            player.runCommand(`title @s actionbar §eYou are §cspawn protected! §l§3(§p${formatTime(spscore)}§3)`);
        }
        if (getScore(player, `spawnkill`) <= 0) {
            player.removeTag(`cl-spawnkill`);
            player.runCommand(`title @s actionbar §3You are no longer spawn protected!`);
        }
        if (getScore(player, `combatlog`) <= 0) {
            player.runCommand(`title @s actionbar §3You are no longer in combat!`);
            player.removeTag(`cl-combatlogged`);
        }
    }
}, 1);
world.beforeEvents.itemUse.subscribe(data => {
    let player = data.source;
    let item = data.itemStack;
    if (player.typeId !== `minecraft:player`)
        return;
    if (item.typeId == `minecraft:ender_pearl` && player.hasTag(`cl-combatlogged`)) {
        if (clSettings.get(`disabletpcombat`) !== `true`)
            return;
        data.cancel = true;
        sendPlayerMessage(player, `§4You are in combat!`);
    }
});
world.afterEvents.playerSpawn.subscribe(({ player, initialSpawn }) => {
    if (initialSpawn == false)
        return;
    if (player.hasTag(`cl-combatlogged`)) {
        player.removeTag(`cl-combatlogged`);
        player.runCommand(`gamemode s @s`);
        if (clSettings.get(`itemdroplog`) == `false`)
            player.runCommand(`clear @s`);
        system.run(() => {
            player.kill();
            sendAllMessage(`§cKilled ${player.name} for §6Combat Logging`);
        });
    }
    sendPlayerMessage(player, `§4-----
§cThis world/realm/server is protected by Anti Combat Log by §lGamerDos§r
§6AntiCL version: v1.1.2-release
Command Prefix: §l${config.prefix}§r
§dSupport the addon creator! https://patreon.com/gamerdos
§1Need help setting up the addon? https://dsc.gg/gamerdos
§4-----`);
    const equipmentInventory = player.getComponent(`equipment_inventory`);
    if (equipmentInventory.getEquipment(EquipmentSlot.chest).typeId === `minecraft:barrier`) {
        equipmentInventory.setEquipment(EquipmentSlot.chest, new ItemStack(`minecraft:air`));
    }
});
