import { Player, world } from "@minecraft/server";

/**
 * AntiCL config, change the staffTag and prefix
 */
export const config = {
    prefix: '==',
    staffTag: 'cl-staff'
};



/**
 * Send a message to a player
 * @param player Player to send the message to
 * @param message Message the player will be sent
 * @param removeSpaces Whether to remove repeated spaces
 */
export function sendPlayerMessage(player: Player, message: string, removeSpaces: boolean = false) {
    if (removeSpaces) {
        player.runCommandAsync(`tellraw @s {"rawtext":[{"text":"${message.replaceAll('"', '\\\"').replaceAll(/(\r\n|\r|\n)/g, `\n`).replaceAll(/  +/g, ``)}"}]}`);
        return;
    }
    else {
        player.runCommandAsync(`tellraw @s {"rawtext":[{"text":"${message.replaceAll('"', '\\\"')}"}]}`);
        return;
    }
}

/**
 * Send every player a message
 * @param player Player that will send the message
 * @param message Message that will be sent
 * @param removeSpaces Whether to remove repeated spaces
 */
export function sendAllMessage(message: string, removeSpaces: boolean = false) {
    if (removeSpaces) {
        world.getDimension(`overworld`).runCommandAsync(`tellraw @a {"rawtext":[{"text":"${message.replaceAll('"', '\\\"').replaceAll(/(\r\n|\r|\n)/g, `\n`).replaceAll(/  +/g, ``)}"}]}`);
        return;
    }
    else {
        world.getDimension(`overworld`).runCommandAsync(`tellraw @a {"rawtext":[{"text":"${message.replaceAll('"', '\\\"')}"}]}`);
        return;
    }
}

/**
 * Send a message to a target
 * @param target Player to send the message to
 * @param message Message the player will be sent
 * @param removeSpaces Whether to remove repeated spaces
 */
export function sendMessage(target: string, message: string, removeSpaces: boolean = false) {
    if (removeSpaces) {
        world.getDimension(`overworld`).runCommandAsync(`tellraw ${target} {"rawtext":[{"text":"${message.replaceAll('"', '\\\"').replaceAll(/(\r\n|\r|\n)/g, `\n`).replaceAll(/  +/g, ``)}"}]}`);
        return;
    }
    else {
        world.getDimension(`overworld`).runCommandAsync(`tellraw ${target} {"rawtext":[{"text":"${message.replaceAll('"', '\\\"')}"}]}`);
        return;
    }
}